@@ -1,6 +1,6 @@
 package DBIx::Custom;
 
-our $VERSION = '0.1630';
+our $VERSION = '0.1633';
 
 use 5.008001;
 use strict;
@@ -13,11 +13,12 @@ use DBI;
 use DBIx::Custom::Result;
 use DBIx::Custom::Query;
 use DBIx::Custom::QueryBuilder;
+use DBIx::Custom::Where;
 use Encode qw/encode_utf8 decode_utf8/;
 
 __PACKAGE__->attr(
-    [qw/data_source dbh dbi_options password user/],
-
+    [qw/data_source dbh password user/],
+    dbi_options => sub { {} },
     cache => 1,
     filters => sub {
         {
@@ -240,7 +241,7 @@ sub create_query {
 }
 
 our %VALID_DELETE_ARGS
-  = map { $_ => 1 } qw/table where append filter allow_delete_all/;
+  = map { $_ => 1 } qw/table where append filter allow_delete_all query/;
 
 sub delete {
     my ($self, %args) = @_;
@@ -278,9 +279,13 @@ sub delete {
     my $source = "delete from $table $where_clause";
     $source .= " $append" if $append;
     
+    # Create query
+    my $query = $self->create_query($source);
+    return $query if $args{query};
+    
     # Execute query
     my $ret_val = $self->execute(
-        $source, param  => $where, filter => $filter,
+        $query, param  => $where, filter => $filter,
         table => $table);
     
     return $ret_val;
@@ -386,7 +391,7 @@ sub expand {
 }
 
 our %VALID_INSERT_ARGS = map { $_ => 1 } qw/table param append
-                                            filter/;
+                                            filter query/;
 sub insert {
     my ($self, %args) = @_;
 
@@ -410,9 +415,13 @@ sub insert {
                . join(' ', @insert_keys) . '}';
     $source .= " $append" if $append;
     
+    # Create query
+    my $query = $self->create_query($source);
+    return $query if $args{query};
+    
     # Execute query
     my $ret_val = $self->execute(
-        $source,
+        $query,
         param  => $param,
         filter => $filter,
         table => $table
@@ -453,6 +462,12 @@ sub new {
     return $self;
 }
 
+sub or {
+    my $self = shift;
+    my $values = ref $_[0] eq 'ARRAY' ? $_[0] : [@_];
+    return DBIx::Custom::Or->new(values => $values);
+}
+
 sub register_filter {
     my $invocant = shift;
     
@@ -464,7 +479,7 @@ sub register_filter {
 }
 
 our %VALID_SELECT_ARGS
-  = map { $_ => 1 } qw/table column where append relation filter/;
+  = map { $_ => 1 } qw/table column where append relation filter query/;
 
 sub select {
     my ($self, %args) = @_;
@@ -507,25 +522,37 @@ sub select {
     
     # Where clause
     my $param;
-    if (ref $where eq 'HASH' && keys %$where) {
+    my $wexists;
+    if (ref $where eq 'HASH') {
         $param = $where;
-        $source .= 'where (';
-        foreach my $where_key (keys %$where) {
-            $source .= "{= $where_key} and ";
+        $wexists = keys %$where;
+        
+        if ($wexists) {
+            $source .= 'where (';
+            foreach my $where_key (keys %$where) {
+                $source .= "{= $where_key} and ";
+            }
+            $source =~ s/ and $//;
+            $source .= ') ';
         }
-        $source =~ s/ and $//;
-        $source .= ') ';
     }
     elsif (ref $where eq 'ARRAY') {
-        my$where_str = $where->[0] || '';
+        my $w = $where->[0] || '';
         $param = $where->[1];
         
-        $source .= "where ($where_str) ";
+        $wexists = $w =~ /\S/;
+        $source .= "where ($w) " if $wexists;
+    }
+    elsif (ref $where eq 'DBIx::Custom::Where') {
+        $param = $where->param;
+        my $w = $where->to_string;
+        $wexists = $w =~ /\S/;
+        $source .= $w;
     }
     
     # Relation
     if ($relation) {
-        $source .= $where ? "and " : "where ";
+        $source .= $wexists ? "and " : "where ";
         foreach my $rkey (keys %$relation) {
             $source .= "$rkey = " . $relation->{$rkey} . " and ";
         }
@@ -535,9 +562,13 @@ sub select {
     # Append some statement
     $source .= " $append" if $append;
     
+    # Create query
+    my $query = $self->create_query($source);
+    return $query if $args{query};
+    
     # Execute query
     my $result = $self->execute(
-        $source, param  => $param, filter => $filter,
+        $query, param  => $param, filter => $filter,
         table => $tables);    
     
     return $result;
@@ -580,7 +611,7 @@ sub txn_scope {
 
 our %VALID_UPDATE_ARGS
   = map { $_ => 1 } qw/table param
-                       where append filter allow_update_all/;
+                       where append filter allow_update_all query/;
 
 sub update {
     my ($self, %args) = @_;
@@ -641,8 +672,12 @@ sub update {
         }
     }
     
+    # Create query
+    my $query = $self->create_query($source);
+    return $query if $args{query};
+    
     # Execute query
-    my $ret_val = $self->execute($source, param  => $param, 
+    my $ret_val = $self->execute($query, param  => $param, 
                                  filter => $filter,
                                  table => $table);
     
@@ -651,6 +686,8 @@ sub update {
 
 sub update_all { shift->update(allow_update_all => 1, @_) };
 
+sub where { DBIx::Custom::Where->new }
+
 sub _build_binds {
     my ($self, $params, $columns, $filter) = @_;
     
@@ -1074,7 +1111,8 @@ This is used in C<select()>
     $dbi->delete(table  => $table,
                  where  => \%where,
                  append => $append,
-                 filter => \%filter);
+                 filter => \%filter,
+                 query  => 1);
 
 Execute delete statement.
 C<delete> method have C<table>, C<where>, C<append>, and C<filter> arguments.
@@ -1082,6 +1120,8 @@ C<table> is a table name.
 C<where> is where clause. this must be hash reference.
 C<append> is a string added at the end of the SQL statement.
 C<filter> is filters when parameter binding is executed.
+C<query> is if you don't execute sql and get L<DBIx::Custom::Query> object as return value.
+default to 0. This is experimental.
 Return value of C<delete()> is the count of affected rows.
 
 B<Example:>
@@ -1127,7 +1167,8 @@ Register helper methods. These method is called from L<DBIx::Custom> object dire
     $dbi->insert(table  => $table, 
                  param  => \%param,
                  append => $append,
-                 filter => \%filter);
+                 filter => \%filter,
+                 query  => 1);
 
 Execute insert statement.
 C<insert> method have C<table>, C<param>, C<append>
@@ -1136,6 +1177,8 @@ C<table> is a table name.
 C<param> is the pairs of column name value. this must be hash reference.
 C<append> is a string added at the end of the SQL statement.
 C<filter> is filters when parameter binding is executed.
+C<query> is if you don't execute sql and get L<DBIx::Custom::Query> object as return value.
+default to 0. This is experimental.
 This is overwrites C<default_bind_filter>.
 Return value of C<insert()> is the count of affected rows.
 
@@ -1166,6 +1209,13 @@ Create a new L<DBIx::Custom> object.
 Iterate all columns of all tables. Argument is callback.
 You can do anything by callback.
 
+=head2 C<(experimental) or>
+
+    $or = $dbi->or(1, 5);
+
+Create L<DBIx::Custom::Or> object. This is used with select method's
+where option.
+
 =head2 C<register_filter>
 
     $dbi->register_filter(%filters);
@@ -1229,7 +1279,8 @@ This is same as L<DBI>'s C<rollback>.
                               where    => \%where,
                               append   => $append,
                               relation => \%relation,
-                              filter   => \%filter);
+                              filter   => \%filter,
+                              query    => 1);
 
 Execute select statement.
 C<select> method have C<table>, C<column>, C<where>, C<append>,
@@ -1238,6 +1289,8 @@ C<table> is a table name.
 C<where> is where clause. this is normally hash reference.
 C<append> is a string added at the end of the SQL statement.
 C<filter> is filters when parameter binding is executed.
+C<query> is if you don't execute sql and get L<DBIx::Custom::Query> object as return value.
+default to 0. This is experimental.
 
 B<Example:>
 
@@ -1283,7 +1336,8 @@ Second element is paramters.
                  param  => \%params,
                  where  => \%where,
                  append => $append,
-                 filter => \%filter)
+                 filter => \%filter,
+                 query  => 1)
 
 Execute update statement.
 C<update> method have C<table>, C<param>, C<where>, C<append>
@@ -1293,6 +1347,8 @@ C<param> is column-value pairs. this must be hash reference.
 C<where> is where clause. this must be hash reference.
 C<append> is a string added at the end of the SQL statement.
 C<filter> is filters when parameter binding is executed.
+C<query> is if you don't execute sql and get L<DBIx::Custom::Query> object as return value.
+default to 0. This is experimental.
 This is overwrites C<default_bind_filter>.
 Return value of C<update()> is the count of affected rows.
 
@@ -1349,6 +1405,12 @@ B<Example:>
                      param  => {author => 'taro'},
                      filter => {author => 'encode_utf8'});
 
+=head2 C<(experimental) where>
+
+    my $where = $dbi->where;
+
+Create a new L<DBIx::Custom::Where> object.
+
 =head2 C<(deprecated) default_bind_filter>
 
     my $default_bind_filter = $dbi->default_bind_filter;
