@@ -7,6 +7,9 @@ use base 'Object::Simple';
 
 use Carp 'croak';
 
+# Carp trust relationship
+push @DBIx::Custom::CARP_NOT, __PACKAGE__;
+
 __PACKAGE__->attr(['dbi', 'name', 'model']);
 
 our $AUTOLOAD;
